@@ -1,4 +1,5 @@
 
+import time
 import logging
 import socket
 
@@ -7,9 +8,11 @@ from twisted.internet.defer import Deferred
 from landscape.lib.twisted_util import gather_results
 from landscape.lib.bpickle import loads
 from landscape.lib.log import log_failure
+from landscape.lib.fetch import fetch, FetchError
 
 
-EC2_API = "http://169.254.169.254/latest"
+EC2_HOST = "169.254.169.254"
+EC2_API = "http://%s/latest" % (EC2_HOST,)
 
 
 class InvalidCredentialsError(Exception):
@@ -33,7 +36,15 @@ def config_property(name):
 
 
 class Identity(object):
-    """Maintains details about the identity of this Landscape client."""
+    """Maintains details about the identity of this Landscape client.
+
+    @ivar secure_id: A server-provided ID for secure message exchange.
+    @ivar insecure_id: Non-secure server-provided ID, mainly used with
+        the ping server.
+    @ivar computer_title: See L{BrokerConfiguration}.
+    @ivar account_name: See L{BrokerConfiguration}.
+    @ivar registration_password: See L{BrokerConfiguration}.
+    """
 
     secure_id = persist_property("secure-id")
     insecure_id = persist_property("insecure-id")
@@ -42,6 +53,11 @@ class Identity(object):
     registration_password = config_property("registration_password")
 
     def __init__(self, config, persist):
+        """
+        @param config: A L{BrokerConfiguration} object, used to set the
+            C{computer_title}, C{account_name} and C{registration_password}
+            instance variables.
+        """
         self._config = config
         self._persist = persist.root_at("registration")
 
@@ -98,36 +114,6 @@ class RegistrationHandler(object):
         self._exchange.exchange()
         return result
 
-    def _extract_ec2_instance_data(self, raw_user_data, launch_index):
-        """
-        Given the raw string of EC2 User Data, parse it and return the dict of
-        instance data for this particular instance.
-
-        If the data can't be parsed, a debug message will be logged and None
-        will be returned.
-        """
-        try:
-            user_data = loads(raw_user_data)
-        except ValueError:
-            logging.debug("Got invalid user-data %r" % (raw_user_data,))
-            return
-
-        if not isinstance(user_data, dict):
-            logging.debug("user-data %r is not a dict" % (user_data,))
-            return
-        for key in "otps", "exchange-url", "ping-url":
-            if key not in user_data:
-                logging.debug("user-data %r doesn't have key %r."
-                              % (user_data, key))
-                return
-        if len(user_data["otps"]) <= launch_index:
-            logging.debug("user-data %r doesn't have OTP for launch index %d"
-                          % (user_data, launch_index))
-            return
-        return {"otp": user_data["otps"][launch_index],
-                "exchange-url": user_data["exchange-url"],
-                "ping-url": user_data["ping-url"]}
-
     def _fetch_ec2_data(self):
         id = self._identity
         if self._cloud and not id.secure_id:
@@ -170,7 +156,7 @@ class RegistrationHandler(object):
                 self._ec2_data["launch_index"] = int(
                     self._ec2_data["launch_index"])
 
-                instance_data = self._extract_ec2_instance_data(
+                instance_data = _extract_ec2_instance_data(
                     raw_user_data, int(launch_index))
                 if instance_data is not None:
                     self._otp = instance_data["otp"]
@@ -191,9 +177,15 @@ class RegistrationHandler(object):
             registration_data.addErrback(log_error)
 
     def _handle_exchange_done(self):
+        """Registered handler for the C{"exchange-done"} event.
+
+        If we are not registered yet, schedule another message exchange.
+
+        The first exchange made us accept the message type "register", so
+        the next "pre-exchange" event will make L{_handle_pre_exchange}
+        queue a registration message for delivery.
+        """
         if self.should_register() and not self._should_register:
-            # We received accepted-types (first exchange), so we now trigger
-            # the second exchange for registration
             self._exchange.exchange()
 
     def _handle_pre_exchange(self):
@@ -218,7 +210,7 @@ class RegistrationHandler(object):
                     logging.info("Queueing message to register with OTP")
                     message = {"type": "register-cloud-vm",
                                "otp": self._otp,
-                               "hostname": socket.gethostname(),
+                               "hostname": socket.getfqdn(),
                                "account_name": None,
                                "registration_password": None,
                                }
@@ -229,7 +221,7 @@ class RegistrationHandler(object):
                                  "%r as an EC2 instance." % (id.account_name,))
                     message = {"type": "register-cloud-vm",
                                "otp": None,
-                               "hostname": socket.gethostname(),
+                               "hostname": socket.getfqdn(),
                                "account_name": id.account_name,
                                "registration_password": \
                                    id.registration_password}
@@ -246,15 +238,18 @@ class RegistrationHandler(object):
                            "computer_title": id.computer_title,
                            "account_name": id.account_name,
                            "registration_password": id.registration_password,
-                           "hostname": socket.gethostname()}
+                           "hostname": socket.getfqdn()}
                 self._exchange.send(message)
             else:
                 self._reactor.fire("registration-failed")
 
     def _handle_set_id(self, message):
-        """
+        """Registered handler for the C{"set-id"} event.
+        
         Record and start using the secure and insecure IDs from the given
         message.
+
+        Fire C{"registration-done"} and C{"resynchronize-clients"}.
         """
         id = self._identity
         id.secure_id = message.get("id")
@@ -301,3 +296,76 @@ class RegistrationResponse(object):
     def _failed(self):
         self.deferred.errback(InvalidCredentialsError())
         self._cancel_calls()
+
+
+def _extract_ec2_instance_data(raw_user_data, launch_index):
+    """
+    Given the raw string of EC2 User Data, parse it and return the dict of
+    instance data for this particular instance.
+
+    If the data can't be parsed, a debug message will be logged and None
+    will be returned.
+    """
+    try:
+        user_data = loads(raw_user_data)
+    except ValueError:
+        logging.debug("Got invalid user-data %r" % (raw_user_data,))
+        return
+
+    if not isinstance(user_data, dict):
+        logging.debug("user-data %r is not a dict" % (user_data,))
+        return
+    for key in "otps", "exchange-url", "ping-url":
+        if key not in user_data:
+            logging.debug("user-data %r doesn't have key %r."
+                          % (user_data, key))
+            return
+    if len(user_data["otps"]) <= launch_index:
+        logging.debug("user-data %r doesn't have OTP for launch index %d"
+                      % (user_data, launch_index))
+        return
+    return {"otp": user_data["otps"][launch_index],
+            "exchange-url": user_data["exchange-url"],
+            "ping-url": user_data["ping-url"]}
+
+
+def _wait_for_network():
+    """
+    Keep trying to connect to the EC2 metadata server until it becomes
+    accessible or until five minutes pass.
+
+    This is necessary because the networking init script on Ubuntu is
+    asynchronous; the network may not actually be up by the time the
+    landscape-client init script is invoked.
+    """
+    timeout = 5*60
+    port = 80
+
+    start = time.time()
+    while True:
+        s = socket.socket()
+        try:
+            s.connect((EC2_HOST, port))
+            s.close()
+            return
+        except socket.error, e:
+            time.sleep(1)
+            if time.time() - start > timeout:
+                break
+                
+
+def is_cloud_managed(fetch=fetch):
+    """
+    Return C{True} if the machine has been started by Landscape, i.e. if we can
+    find the expected data inside the EC2 user-data field.
+    """
+    _wait_for_network()
+    try:
+        raw_user_data = fetch(EC2_API + "/user-data",
+                              connect_timeout=5)
+        launch_index = fetch(EC2_API + "/meta-data/ami-launch-index",
+                             connect_timeout=5)
+    except FetchError:
+        return False
+    instance_data = _extract_ec2_instance_data(raw_user_data, int(launch_index))
+    return instance_data is not None
