@@ -6,8 +6,10 @@ import pycurl
 
 from twisted.internet.threads import deferToThread
 
+class FetchError(Exception):
+    pass
 
-class HTTPCodeError(Exception):
+class HTTPCodeError(FetchError):
 
     def __init__(self, http_code, body):
         self.http_code = http_code
@@ -20,7 +22,20 @@ class HTTPCodeError(Exception):
         return "<HTTPCodeError http_code=%d>" % self.http_code
 
 
-def fetch(url, post=False, data="", headers={}, cainfo=None, curl=None):
+class PyCurlError(FetchError):
+    def __init__(self, error_code, message):
+        self.error_code = error_code
+        self.message = message
+
+    def __str__(self):
+        return "Error %d: %s" % (self.error_code, self.message)
+
+    def __repr__(self):
+        return "<PyCurlError args=(%d, '%s')>" % (self.error_code,
+                                                  self.message)
+
+def fetch(url, post=False, data="", headers={}, cainfo=None, curl=None,
+          connect_timeout=30, total_timeout=600):
     """Retrieve a URL and return the content.
 
     @param url: The url to be fetched.
@@ -53,12 +68,16 @@ def fetch(url, post=False, data="", headers={}, cainfo=None, curl=None):
     curl.setopt(pycurl.URL, url)
     curl.setopt(pycurl.FOLLOWLOCATION, True)
     curl.setopt(pycurl.MAXREDIRS, 5)
-    curl.setopt(pycurl.CONNECTTIMEOUT, 30)
-    curl.setopt(pycurl.TIMEOUT, 600)
+    curl.setopt(pycurl.CONNECTTIMEOUT, connect_timeout)
+    curl.setopt(pycurl.LOW_SPEED_LIMIT, 1)
+    curl.setopt(pycurl.LOW_SPEED_TIME, total_timeout)
     curl.setopt(pycurl.NOSIGNAL, 1)
     curl.setopt(pycurl.WRITEFUNCTION, input.write)
 
-    curl.perform()
+    try:
+        curl.perform()
+    except pycurl.error, e:
+        raise PyCurlError(e.args[0], e.args[1])
 
     body = input.getvalue()
 
