@@ -34,8 +34,9 @@ def get_user_info(username=None):
     gid = None
     path = None
     if username is not None:
+        username_str = username.encode("utf-8")
         try:
-            info = pwd.getpwnam(username)
+            info = pwd.getpwnam(username_str)
         except KeyError, e:
             raise UnknownUserError(u"Unknown user '%s'" % username)
         uid = info.pw_uid
@@ -137,7 +138,9 @@ class ScriptExecutionPlugin(ManagerPlugin, ScriptRunnerMixin):
     def _respond(self, status, data, opid, result_code=None):
         message =  {"type": "operation-result",
                     "status": status,
-                    "result-text": data,
+                    # Let's decode result-text, replacing non-printable
+                    # characters
+                    "result-text": data.decode("utf-8", "replace"),
                     "operation-id": opid}
         if result_code:
             message["result-code"] = result_code
