@@ -3,6 +3,7 @@ import os
 import sys
 import tempfile
 import stat
+import pwd
 
 from twisted.internet.defer import gatherResults
 from twisted.internet.error import ProcessDone
@@ -19,7 +20,7 @@ from landscape.tests.mocker import ANY, ARGS
 
 
 def get_default_environment():
-    username = os.getlogin()
+    username = pwd.getpwuid(os.getuid())[0]
     uid, gid, home = get_user_info(username)
     return  {
         "PATH": UBUNTU_PATH,
@@ -334,11 +335,9 @@ class RunScriptTests(LandscapeTest):
         protocol.childDataReceived(1, "hi")
         protocol.processEnded(Failure(ProcessDone(0)))
         self.manager.reactor.advance(501)
-        def got_error(f):
-            print f
-            self.assertTrue(f.check(ProcessTimeLimitReachedError))
-            self.assertEquals(f.value.data, "hi\n")
-        result.addErrback(got_error)
+        def got_result(output):
+            self.assertEquals(output, "hi")
+        result.addCallback(got_result)
         return result
 
     def test_script_is_owned_by_user(self):
@@ -348,7 +347,7 @@ class RunScriptTests(LandscapeTest):
         correct permissions. Therefore os.chmod and os.chown must be called
         before data is written.
         """
-        username = os.getlogin()
+        username = pwd.getpwuid(os.getuid())[0]
         uid, gid, home = get_user_info(username)
 
         mock_chown = self.mocker.replace("os.chown", passthrough=False)
@@ -474,7 +473,7 @@ class ScriptExecutionMessageTests(LandscapeIsolatedTest):
 
     def test_user(self):
         """A user can be specified in the message."""
-        username = os.getlogin()
+        username = pwd.getpwuid(os.getuid())[0]
         uid, gid, home = get_user_info(username)
 
         # ignore the call to chown!
@@ -551,7 +550,7 @@ class ScriptExecutionMessageTests(LandscapeIsolatedTest):
 
     def test_urgent_response(self):
         """Responses to script execution messages are urgent."""
-        username = os.getlogin()
+        username = pwd.getpwuid(os.getuid())[0]
         uid, gid, home = get_user_info(username)
 
         # ignore the call to chown!
@@ -585,6 +584,43 @@ class ScriptExecutionMessageTests(LandscapeIsolatedTest):
         result.addCallback(got_result)
         return result
 
+    def test_binary_output(self):
+        """
+        If a script outputs non-printable characters not handled by utf-8, they
+        are replaced during the encoding phase but the script succeeds.
+        """
+        username = pwd.getpwuid(os.getuid())[0]
+        uid, gid, home = get_user_info(username)
+
+        mock_chown = self.mocker.replace("os.chown", passthrough=False)
+        mock_chown(ARGS)
+
+        def spawn_called(protocol, filename, uid, gid, path, env):
+            protocol.childDataReceived(1,
+            "\x7fELF\x01\x01\x01\x00\x00\x00\x95\x01")
+            protocol.processEnded(Failure(ProcessDone(0)))
+            self._verify_script(filename, sys.executable, "print 'hi'")
+        process_factory = self.mocker.mock()
+        process_factory.spawnProcess(
+            ANY, ANY, uid=uid, gid=gid, path=ANY,
+            env=get_default_environment())
+        self.mocker.call(spawn_called)
+
+        self.mocker.replay()
+
+        self.manager.add(ScriptExecutionPlugin(process_factory=process_factory))
+
+        def got_result(r):
+            self.assertTrue(self.broker_service.exchanger.is_urgent())
+            [message] = self.broker_service.message_store.get_pending_messages()
+            self.assertEquals(
+                message["result-text"],
+                 u"\x7fELF\x01\x01\x01\x00\x00\x00\ufffd\x01")
+
+        result = self._send_script(sys.executable, "print 'hi'")
+        result.addCallback(got_result)
+        return result
+
     def test_parse_error_causes_operation_failure(self):
         """
         If there is an error parsing the message, an operation-result will be
@@ -596,12 +632,20 @@ class ScriptExecutionMessageTests(LandscapeIsolatedTest):
         self.manager.dispatch_message(
             {"type": "execute-script", "operation-id": 444})
 
+        if sys.version_info[:2] < (2, 6):
+            expected_message = [{"type": "operation-result",
+                                 "operation-id": 444,
+                                 "result-text": u"KeyError: 'username'",
+                                 "status": FAILED}]
+        else:
+            expected_message = [{"type": "operation-result",
+                                 "operation-id": 444,
+                                 "result-text": u"KeyError: username",
+                                 "status": FAILED}]
+
         self.assertMessages(
             self.broker_service.message_store.get_pending_messages(),
-            [{"type": "operation-result",
-              "operation-id": 444,
-              "result-text": u"KeyError: 'username'",
-              "status": FAILED}])
+            expected_message)
 
         self.assertTrue("KeyError: 'username'" in self.logfile.getvalue())
 
