@@ -5,8 +5,8 @@ from landscape.tests.helpers import LandscapeTest, MakePathHelper, MonitorHelper
 from landscape.tests.mocker import ANY
 
 
-def get_hostname():
-    return "ooga"
+def get_fqdn():
+    return "ooga.local"
 
 
 class ComputerInfoTest(LandscapeTest):
@@ -48,16 +48,16 @@ DISTRIB_CODENAME=dapper
 DISTRIB_DESCRIPTION="Ubuntu 6.06.1 LTS"
 """)
 
-    def test_get_hostname(self):
+    def test_get_fqdn(self):
         self.mstore.set_accepted_types(["computer-info"])
-        plugin = ComputerInfo(get_hostname=get_hostname,
+        plugin = ComputerInfo(get_fqdn=get_fqdn,
                               lsb_release_filename=self.lsb_release_filename)
         self.monitor.add(plugin)
         plugin.exchange()
         messages = self.mstore.get_pending_messages()
         self.assertEquals(len(messages), 1)
         self.assertEquals(messages[0]["type"], "computer-info")
-        self.assertEquals(messages[0]["hostname"], "ooga")
+        self.assertEquals(messages[0]["hostname"], "ooga.local")
 
     def test_get_real_hostname(self):
         self.mstore.set_accepted_types(["computer-info"])
@@ -72,7 +72,7 @@ DISTRIB_DESCRIPTION="Ubuntu 6.06.1 LTS"
 
     def test_only_report_changed_hostnames(self):
         self.mstore.set_accepted_types(["computer-info"])
-        plugin = ComputerInfo(get_hostname=get_hostname)
+        plugin = ComputerInfo(get_fqdn=get_fqdn)
         self.monitor.add(plugin)
         plugin.exchange()
         messages = self.mstore.get_pending_messages()
@@ -89,7 +89,7 @@ DISTRIB_DESCRIPTION="Ubuntu 6.06.1 LTS"
                 i = i + 1
 
         self.mstore.set_accepted_types(["computer-info"])
-        plugin = ComputerInfo(get_hostname=hostname_factory().next)
+        plugin = ComputerInfo(get_fqdn=hostname_factory().next)
         self.monitor.add(plugin)
 
         plugin.exchange()
@@ -270,14 +270,14 @@ DISTRIB_NEW_UNEXPECTED_KEY=ooga
         """
         self.mstore.set_accepted_types(["distribution-info", "computer-info"])
         meminfo_filename = self.make_path(self.sample_memory_info)
-        plugin = ComputerInfo(get_hostname=get_hostname,
+        plugin = ComputerInfo(get_fqdn=get_fqdn,
                               meminfo_file=meminfo_filename,
                               lsb_release_filename=self.lsb_release_filename)
         self.monitor.add(plugin)
         plugin.exchange()
         self.reactor.fire("resynchronize")
         plugin.exchange()
-        computer_info = {"type": "computer-info", "hostname": "ooga",
+        computer_info = {"type": "computer-info", "hostname": "ooga.local",
                          "timestamp": 0, "total-memory": 1510,
                          "total-swap": 1584}
         dist_info = {"type": "distribution-info",
