@@ -3,8 +3,9 @@ import time
 import sys
 import os
 import pwd
+import grp
 
-from twisted.internet.defer import succeed, fail
+from twisted.internet.defer import fail
 
 from landscape.package.reporter import find_reporter_command
 from landscape.package.taskhandler import PackageTaskHandler, run_task_handler
@@ -42,14 +43,24 @@ class PackageChanger(PackageTaskHandler):
 
     def run(self):
         task1 = self._store.get_next_task(self.queue_name)
-        result = super(PackageChanger, self).run()
+
         def finished(result):
             task2 = self._store.get_next_task(self.queue_name)
             if task1 and task1.id != (task2 and task2.id):
+                # In order to let the reporter run smart-update cleanly,
+                # we have to deinitialize Smart, so that the write lock
+                # gets released
+                self._facade.deinit()
                 if os.getuid() == 0:
+                    os.setgid(grp.getgrnam("landscape").gr_gid)
                     os.setuid(pwd.getpwnam("landscape").pw_uid)
                 os.system(find_reporter_command())
-        return result.addCallback(finished)
+
+        result = self.use_hash_id_db()
+        result.addCallback(lambda x: self.handle_tasks())
+        result.addCallback(finished)
+
+        return result
 
     def handle_tasks(self):
         result = super(PackageChanger, self).handle_tasks()
