@@ -7,14 +7,12 @@ from twisted.internet.defer import Deferred
 
 from smart.cache import Provides
 
-from landscape.lib.lock import lock_path
-
 from landscape.package.changer import (
     PackageChanger, main, find_changer_command, UNKNOWN_PACKAGE_DATA_TIMEOUT)
 from landscape.package.store import PackageStore
 from landscape.package.facade import (
-    SmartFacade, DependencyError, TransactionError, SmartError)
-from landscape.broker.remote import RemoteBroker
+    DependencyError, TransactionError, SmartError)
+from landscape.deployment import Configuration
 
 from landscape.tests.mocker import ANY
 from landscape.tests.helpers import (
@@ -31,7 +29,8 @@ class PackageChangerTest(LandscapeIsolatedTest):
         super(PackageChangerTest, self).setUp()
 
         self.store = PackageStore(self.makeFile())
-        self.changer = PackageChanger(self.store, self.facade, self.remote)
+        self.config = Configuration()
+        self.changer = PackageChanger(self.store, self.facade, self.remote, self.config)
 
         service = self.broker_service
         service.message_store.set_accepted_types(["change-packages-result"])
@@ -420,18 +419,40 @@ class PackageChangerTest(LandscapeIsolatedTest):
                               "REPORTER RUN")
         return result.addCallback(got_result)
 
-    def test_set_effective_uid_when_running_as_root(self):
+    def test_set_effective_uid_and_gid_when_running_as_root(self):
         """
         After the package changer has run, we want the package-reporter to run
         to report the recent changes.  If we're running as root, we want to
-        change to the "landscape" user.
+        change to the "landscape" user and "landscape" group. We also want to
+        deinitialize Smart to let the reporter run smart-update cleanly.
         """
+
         # We are running as root
         getuid_mock = self.mocker.replace("os.getuid")
         getuid_mock()
         self.mocker.result(0)
 
-        # We want to return a known uid
+        # The order matters (first smart then gid and finally uid)
+        self.mocker.order()
+
+        # Deinitialize smart
+        facade_mock = self.mocker.patch(self.facade)
+        facade_mock.deinit()
+
+        # We want to return a known gid
+        grnam_mock = self.mocker.replace("grp.getgrnam")
+        grnam_mock("landscape")
+
+        class FakeGroup(object):
+            gr_gid = 199
+
+        self.mocker.result(FakeGroup())
+
+        # First the changer should change the group
+        setgid_mock = self.mocker.replace("os.setgid")
+        setgid_mock(199)
+
+        # And a known uid as well
         pwnam_mock = self.mocker.replace("pwd.getpwnam")
         pwnam_mock("landscape")
 
@@ -440,11 +461,11 @@ class PackageChangerTest(LandscapeIsolatedTest):
 
         self.mocker.result(FakeUser())
 
-        # And now, the changer should change the user
+        # And now the user as well
         setuid_mock = self.mocker.replace("os.setuid")
         setuid_mock(199)
 
-        # Finally, we don't really want the package changer to run.
+        # Finally, we don't really want the package reporter to run.
         system_mock = self.mocker.replace("os.system")
         system_mock(ANY)
 
@@ -457,6 +478,29 @@ class PackageChangerTest(LandscapeIsolatedTest):
         return self.changer.run()
 
 
+    def test_run(self):
+        changer_mock = self.mocker.patch(self.changer)
+
+        self.mocker.order()
+
+        results = [Deferred() for i in range(2)]
+
+        changer_mock.use_hash_id_db()
+        self.mocker.result(results[0])
+
+        changer_mock.handle_tasks()
+        self.mocker.result(results[1])
+
+        self.mocker.replay()
+
+        self.changer.run()
+
+        # It must raise an error because deferreds weren't yet fired.
+        self.assertRaises(AssertionError, self.mocker.verify)
+
+        for deferred in reversed(results):
+            deferred.callback(None)
+
     def test_dont_spawn_reporter_after_running_if_nothing_done(self):
         output_filename = self.makeFile("REPORTER NOT RUN")
         reporter_filename = self.makeFile("#!/bin/sh\necho REPORTER RUN > %s" %
@@ -478,7 +522,6 @@ class PackageChangerTest(LandscapeIsolatedTest):
         return result.addCallback(got_result)
 
     def test_main(self):
-        data_path = self.makeDir()
         self.mocker.order()
         
         run_task_handler = self.mocker.replace("landscape.package.taskhandler"
