@@ -1,14 +1,21 @@
-import gdbm
 import time
 import os
+import re
+import sys
 
 from smart.control import Control
 from smart.cache import Provides
+from smart.const import NEVER
+
+from twisted.internet import reactor
+from twisted.internet.defer import Deferred
+from twisted.internet.utils import getProcessOutputAndValue
 
 import smart
 
 from landscape.package.facade import (
-    SmartFacade, TransactionError, DependencyError, SmartError)
+    TransactionError, DependencyError, ChannelError, SmartError,
+    make_apt_deb_channel, make_deb_dir_channel)
 
 from landscape.tests.mocker import ANY
 from landscape.tests.helpers import LandscapeTest
@@ -79,6 +86,11 @@ class SmartFacadeTest(LandscapeTest):
         pkg = self.facade.get_packages_by_name("name2")[0]
         self.assertEquals(self.facade.get_package_hash(pkg), HASH2)
 
+    def test_get_package_hashes(self):
+        self.facade.reload_channels()
+        hashes = self.facade.get_package_hashes()
+        self.assertEquals(sorted(hashes), sorted([HASH1, HASH2, HASH3]))
+
     def test_get_package_by_hash(self):
         self.facade.reload_channels()
         pkg = self.facade.get_package_by_hash(HASH1)
@@ -91,7 +103,6 @@ class SmartFacadeTest(LandscapeTest):
     def test_reload_channels_clears_hash_cache(self):
         # Load hashes.
         self.facade.reload_channels()
-        start = time.time()
 
         # Hold a reference to packages.
         pkg1 = self.facade.get_packages_by_name("name1")[0]
@@ -393,3 +404,104 @@ class SmartFacadeTest(LandscapeTest):
 
         self.facade.reload_channels()
         self.assertEquals(self.facade.get_package_hash(pkg), None)
+
+    def test_reset_add_get_channels(self):
+
+        channels = [("alias0", {"type": "test"}),
+                    ("alias1", {"type": "test"})]
+
+        self.facade.reset_channels()
+
+        self.assertEquals(self.facade.get_channels(), {})
+
+        self.facade.add_channel(*channels[0])
+        self.facade.add_channel(*channels[1])
+
+        self.assertEquals(self.facade.get_channels(), dict(channels))
+
+    def test_make_channels(self):
+
+        channel0 = make_apt_deb_channel("http://my.url/dir", "hardy", "main")
+        channel1 = make_deb_dir_channel("/my/repo")
+
+        self.assertEquals(channel0, {"baseurl": "http://my.url/dir",
+                                     "distribution": "hardy",
+                                     "components": "main",
+                                     "type": "apt-deb"})
+
+        self.assertEquals(channel1, {"path": "/my/repo",
+                                     "type": "deb-dir"})
+
+    def test_get_arch(self):
+        """
+        The L{SmartFacade.get_arch} should return the system dpkg
+        architecture.
+        """
+        deferred = Deferred()
+
+        def do_test():
+            result = getProcessOutputAndValue("/usr/bin/dpkg",
+                                              ("--print-architecture",))
+            def callback((out, err, code)):
+                self.assertEquals(self.facade.get_arch(), out.strip())
+            result.addCallback(callback)
+            result.chainDeferred(deferred)
+
+        reactor.callWhenRunning(do_test)
+        return deferred
+
+    def test_set_arch_multiple_times(self):
+
+        repository_dir = os.path.join(os.path.dirname(__file__), "repository")
+
+        alias = "alias"
+        channel = {"baseurl": "file://%s" % repository_dir,
+                   "distribution": "hardy",
+                   "components": "main",
+                   "type": "apt-deb"}
+
+        self.facade.set_arch("i386")
+        self.facade.reset_channels()
+        self.facade.add_channel(alias, channel)
+        self.facade.reload_channels()
+
+        pkgs = self.facade.get_packages()
+        self.assertEquals(len(pkgs), 1)
+        self.assertEquals(pkgs[0].name, "syslinux")
+
+        self.facade.deinit()
+        self.facade.set_arch("amd64")
+        self.facade.reset_channels()
+        self.facade.add_channel(alias, channel)
+        self.facade.reload_channels()
+
+        pkgs = self.facade.get_packages()
+        self.assertEquals(len(pkgs), 1)
+        self.assertEquals(pkgs[0].name, "libclthreads2")
+
+    def test_set_caching_with_reload_error(self):
+
+        alias = "alias"
+        channel = {"type": "deb-dir",
+                   "path": "/does/not/exist"}
+
+        self.facade.reset_channels()
+        self.facade.add_channel(alias, channel)
+        self.facade.set_caching(NEVER)
+
+        self.assertRaises(ChannelError, self.facade.reload_channels)
+        self.facade._channels = {}
+
+        ignore_re = re.compile("\[Smart\].*'alias'.*/does/not/exist")
+
+        self.log_helper.ignored_exception_regexes = [ignore_re]
+
+    def test_init_landscape_plugins(self):
+        """
+        The landscape plugin which helps managing proxies is loaded when smart
+        is initialized: this sets a smart configuration variable and load the
+        module.
+        """
+        self.facade.reload_channels()
+        self.assertTrue(smart.sysconf.get("use-landscape-proxies"))
+        self.assertIn("smart.plugins.landscape", sys.modules)
