@@ -1,10 +1,13 @@
 import glob
 import sys
 import os
+import logging
 
 from twisted.internet.defer import Deferred
+from twisted.internet import reactor
 
-from landscape.lib.lock import lock_path
+from landscape.lib.fetch import fetch_async, FetchError
+from landscape.lib.command import CommandError
 
 from landscape.package.store import PackageStore, UnknownHashIDRequest
 from landscape.package.reporter import (
@@ -12,6 +15,7 @@ from landscape.package.reporter import (
 from landscape.package import reporter
 from landscape.package.facade import SmartFacade
 
+from landscape.deployment import Configuration
 from landscape.broker.remote import RemoteBroker
 
 from landscape.package.tests.helpers import (
@@ -30,7 +34,8 @@ class PackageReporterTest(LandscapeIsolatedTest):
         super(PackageReporterTest, self).setUp()
 
         self.store = PackageStore(self.makeFile())
-        self.reporter = PackageReporter(self.store, self.facade, self.remote)
+        self.config = Configuration()
+        self.reporter = PackageReporter(self.store, self.facade, self.remote, self.config)
 
     def set_pkg2_upgrades_pkg1(self):
         previous = self.Facade.channels_reloaded
@@ -219,6 +224,407 @@ class PackageReporterTest(LandscapeIsolatedTest):
         deferred = self.reporter.handle_tasks()
         return deferred.addCallback(got_result)
 
+    def test_fetch_hash_id_db(self):
+
+        # Assume package_hash_id_url is set
+        self.config.data_path = self.makeDir()
+        self.config.package_hash_id_url = "http://fake.url/path/"
+        os.makedirs(os.path.join(self.config.data_path, "package", "hash-id"))
+        hash_id_db_filename = os.path.join(self.config.data_path, "package",
+                                           "hash-id", "uuid_codename_arch")
+
+        # Fake uuid, codename and arch
+        message_store = self.broker_service.message_store
+        message_store.set_server_uuid("uuid")
+        command_mock = self.mocker.replace("landscape.lib.command.run_command")
+        command_mock("lsb_release -cs")
+        self.mocker.result("codename")
+        self.facade.set_arch("arch")
+
+        # Let's say fetch_async is successful
+        hash_id_db_url = self.config.package_hash_id_url + "uuid_codename_arch"
+        fetch_async_mock = self.mocker.replace("landscape.lib.fetch.fetch_async")
+        fetch_async_mock(hash_id_db_url, cainfo=None)
+        fetch_async_result = Deferred()
+        fetch_async_result.callback("hash-ids")
+        self.mocker.result(fetch_async_result)
+
+        # The download should be properly logged
+        logging_mock = self.mocker.replace("logging.info")
+        logging_mock("Downloaded hash=>id database from %s" % hash_id_db_url)
+        self.mocker.result(None)
+
+        # We don't have our hash=>id database yet
+        self.assertFalse(os.path.exists(hash_id_db_filename))
+
+        # Now go!
+        self.mocker.replay()
+        result = self.reporter.fetch_hash_id_db()
+
+        # Check the database
+        def callback(ignored):
+            self.assertTrue(os.path.exists(hash_id_db_filename))
+            self.assertEquals(open(hash_id_db_filename).read(), "hash-ids")
+        result.addCallback(callback)
+
+        return result
+
+    def test_fetch_hash_id_db_does_not_download_twice(self):
+
+        # Let's say that the hash=>id database is already there
+        self.config.package_hash_id_url = "http://fake.url/path/"
+        self.config.data_path = self.makeDir()
+        os.makedirs(os.path.join(self.config.data_path, "package", "hash-id"))
+        hash_id_db_filename = os.path.join(self.config.data_path, "package",
+                                           "hash-id", "uuid_codename_arch")
+        open(hash_id_db_filename, "w").write("test")
+
+        # Fake uuid, codename and arch
+        message_store = self.broker_service.message_store
+        message_store.set_server_uuid("uuid")
+        command_mock = self.mocker.replace("landscape.lib.command.run_command")
+        command_mock("lsb_release -cs")
+        self.mocker.result("codename")
+        self.facade.set_arch("arch")
+
+        # Intercept any call to fetch_async
+        fetch_async_mock = self.mocker.replace("landscape.lib.fetch.fetch_async")
+        fetch_async_mock(ANY)
+
+        # Go!
+        self.mocker.replay()
+        result = self.reporter.fetch_hash_id_db()
+
+        def callback(ignored):
+            # Check that fetch_async hasn't been called
+            self.assertRaises(AssertionError, self.mocker.verify)
+            fetch_async(None)
+
+            # The hash=>id database is still there
+            self.assertEquals(open(hash_id_db_filename).read(), "test")
+
+        result.addCallback(callback)
+
+        return result
+
+    def test_fetch_hash_id_db_undetermined_server_uuid(self):
+        """
+        If the server-uuid can't be determined for some reason, no download
+        should be attempted and the failure should be properly logged.
+        """
+        message_store = self.broker_service.message_store
+        message_store.set_server_uuid(None)
+
+        logging_mock = self.mocker.replace("logging.warning")
+        logging_mock("Couldn't determine which hash=>id database to use: "
+                     "server UUID not available")
+        self.mocker.result(None)
+        self.mocker.replay()
+
+        result = self.reporter.fetch_hash_id_db()
+        return result
+
+    def test_fetch_hash_id_db_undetermined_codename(self):
+
+        # Fake uuid
+        message_store = self.broker_service.message_store
+        message_store.set_server_uuid("uuid")
+
+        # Undetermined codename
+        command_mock = self.mocker.replace("landscape.lib.command.run_command")
+        command_mock("lsb_release -cs")
+        command_error = CommandError("lsb_release -cs", 1, "error")
+        self.mocker.throw(command_error)
+
+        # The failure should be properly logged
+        logging_mock = self.mocker.replace("logging.warning")
+        logging_mock("Couldn't determine which hash=>id database to use: %s" %
+                     str(command_error))
+        self.mocker.result(None)
+
+        # Go!
+        self.mocker.replay()
+        result = self.reporter.fetch_hash_id_db()
+
+        return result
+
+    def test_fetch_hash_id_db_undetermined_arch(self):
+
+        # Fake uuid and codename
+        message_store = self.broker_service.message_store
+        message_store.set_server_uuid("uuid")
+        command_mock = self.mocker.replace("landscape.lib.command.run_command")
+        command_mock("lsb_release -cs")
+        self.mocker.result("codename")
+
+        # Undetermined arch
+        self.facade.set_arch(None)
+
+        # The failure should be properly logged
+        logging_mock = self.mocker.replace("logging.warning")
+        logging_mock("Couldn't determine which hash=>id database to use: "\
+                     "unknown dpkg architecture")
+        self.mocker.result(None)
+
+        # Go!
+        self.mocker.replay()
+        result = self.reporter.fetch_hash_id_db()
+
+        return result
+
+    def test_fetch_hash_id_db_with_default_url(self):
+
+        # Let's say package_hash_id_url is not set but url is
+        self.config.data_path = self.makeDir()
+        self.config.package_hash_id_url = None
+        self.config.url = "http://fake.url/path/message-system/"
+        os.makedirs(os.path.join(self.config.data_path, "package", "hash-id"))
+        hash_id_db_filename = os.path.join(self.config.data_path, "package",
+                                           "hash-id", "uuid_codename_arch")
+
+        # Fake uuid, codename and arch
+        message_store = self.broker_service.message_store
+        message_store.set_server_uuid("uuid")
+        command_mock = self.mocker.replace("landscape.lib.command.run_command")
+        command_mock("lsb_release -cs")
+        self.mocker.result("codename")
+        self.facade.set_arch("arch")
+
+        # Check fetch_async is called with the default url
+        hash_id_db_url = "http://fake.url/path/hash-id-databases/" \
+                         "uuid_codename_arch"
+        fetch_async_mock = self.mocker.replace("landscape.lib.fetch.fetch_async")
+        fetch_async_mock(hash_id_db_url, cainfo=None)
+        fetch_async_result = Deferred()
+        fetch_async_result.callback("hash-ids")
+        self.mocker.result(fetch_async_result)
+
+        # Now go!
+        self.mocker.replay()
+        result = self.reporter.fetch_hash_id_db()
+
+        # Check the database
+        def callback(ignored):
+            self.assertTrue(os.path.exists(hash_id_db_filename))
+            self.assertEquals(open(hash_id_db_filename).read(), "hash-ids")
+        result.addCallback(callback)
+        return result
+
+    def test_fetch_hash_id_db_with_download_error(self):
+
+        # Assume package_hash_id_url is set
+        self.config.data_path = self.makeDir()
+        self.config.package_hash_id_url = "http://fake.url/path/"
+
+        # Fake uuid, codename and arch
+        message_store = self.broker_service.message_store
+        message_store.set_server_uuid("uuid")
+        command_mock = self.mocker.replace("landscape.lib.command.run_command")
+        command_mock("lsb_release -cs")
+        self.mocker.result("codename")
+        self.facade.set_arch("arch")
+
+        # Let's say fetch_async fails
+        hash_id_db_url = self.config.package_hash_id_url + "uuid_codename_arch"
+        fetch_async_mock = self.mocker.replace("landscape.lib.fetch.fetch_async")
+        fetch_async_mock(hash_id_db_url, cainfo=None)
+        fetch_async_result = Deferred()
+        fetch_async_result.errback(FetchError("fetch error"))
+        self.mocker.result(fetch_async_result)
+
+        # The failure should be properly logged
+        logging_mock = self.mocker.replace("logging.warning")
+        logging_mock("Couldn't download hash=>id database: fetch error")
+        self.mocker.result(None)
+
+        # Now go!
+        self.mocker.replay()
+        result = self.reporter.fetch_hash_id_db()
+
+        # We shouldn't have any hash=>id database
+        def callback(ignored):
+            hash_id_db_filename = os.path.join(self.config.data_path, "package",
+                                               "hash-id", "uuid_codename_arch")
+            self.assertEquals(os.path.exists(hash_id_db_filename), False)
+        result.addCallback(callback)
+
+        return result
+
+    def test_fetch_hash_id_db_with_undetermined_url(self):
+
+        # We don't know where to fetch the hash=>id database from
+        self.config.url = None
+        self.config.package_hash_id_url = None
+
+        # Fake uuid, codename and arch
+        message_store = self.broker_service.message_store
+        message_store.set_server_uuid("uuid")
+        command_mock = self.mocker.replace("landscape.lib.command.run_command")
+        command_mock("lsb_release -cs")
+        self.mocker.result("codename")
+        self.facade.set_arch("arch")
+
+        # The failure should be properly logged
+        logging_mock = self.mocker.replace("logging.warning")
+        logging_mock("Can't determine the hash=>id database url")
+        self.mocker.result(None)
+
+        # Let's go
+        self.mocker.replay()
+
+        result = self.reporter.fetch_hash_id_db()
+
+        # We shouldn't have any hash=>id database
+        def callback(ignored):
+            hash_id_db_filename = os.path.join(self.config.data_path, "package",
+                                               "hash-id", "uuid_codename_arch")
+            self.assertEquals(os.path.exists(hash_id_db_filename), False)
+        result.addCallback(callback)
+
+        return result
+
+    def test_fetch_hash_id_db_with_custom_certificate(self):
+        """
+        The L{PackageReporter.fetch_hash_id_db} method takes into account the
+        possible custom SSL certificate specified in the client configuration.
+        """
+
+        self.config.url = "http://fake.url/path/message-system/"
+        self.config.ssl_public_key = "/some/key"
+
+        # Fake uuid, codename and arch
+        message_store = self.broker_service.message_store
+        message_store.set_server_uuid("uuid")
+        command_mock = self.mocker.replace("landscape.lib.command.run_command")
+        command_mock("lsb_release -cs")
+        self.mocker.result("codename")
+        self.facade.set_arch("arch")
+
+        # Check fetch_async is called with the default url
+        hash_id_db_url = "http://fake.url/path/hash-id-databases/" \
+                         "uuid_codename_arch"
+        fetch_async_mock = self.mocker.replace("landscape.lib.fetch.fetch_async")
+        fetch_async_mock(hash_id_db_url, cainfo=self.config.ssl_public_key)
+        fetch_async_result = Deferred()
+        fetch_async_result.callback("hash-ids")
+        self.mocker.result(fetch_async_result)
+
+        # Now go!
+        self.mocker.replay()
+        result = self.reporter.fetch_hash_id_db()
+
+        return result
+
+    def test_run_smart_update(self):
+        """
+        The L{PackageReporter.run_smart_update} method should run smart-update
+        with the proper arguments.
+        """
+        self.reporter.smart_update_filename = self.makeFile(
+            "#!/bin/sh\necho -n $@")
+        os.chmod(self.reporter.smart_update_filename, 0755)
+        logging_mock = self.mocker.replace("logging.debug")
+        logging_mock("'%s' exited with status 0 (out='--after %d', err=''" % (
+            self.reporter.smart_update_filename,
+            self.reporter.smart_update_interval))
+        self.mocker.replay()
+        deferred = Deferred()
+
+        def do_test():
+            def raiseme(x):
+                raise Exception
+            logging.warning = raiseme
+            result = self.reporter.run_smart_update()
+            def callback((out, err, code)):
+                interval = self.reporter.smart_update_interval
+                self.assertEquals(out, "--after %d" % interval)
+                self.assertEquals(err, "")
+                self.assertEquals(code, 0)
+            result.addCallback(callback)
+            result.chainDeferred(deferred)
+
+        reactor.callWhenRunning(do_test)
+        return deferred
+
+    def test_run_smart_update_warns_about_failures(self):
+        """
+        The L{PackageReporter.run_smart_update} method should log a warning
+        in case smart-update terminates with a non-zero exit code other than 1.
+        """
+        self.reporter.smart_update_filename = self.makeFile(
+            "#!/bin/sh\necho -n error >&2\necho -n output\nexit 2")
+        os.chmod(self.reporter.smart_update_filename, 0755)
+        logging_mock = self.mocker.replace("logging.warning")
+        logging_mock("'%s' exited with status 2"
+                     " (error)" % self.reporter.smart_update_filename)
+        self.mocker.replay()
+        deferred = Deferred()
+
+        def do_test():
+            result = self.reporter.run_smart_update()
+            def callback((out, err, code)):
+                interval = self.reporter.smart_update_interval
+                self.assertEquals(out, "output")
+                self.assertEquals(err, "error")
+                self.assertEquals(code, 2)
+            result.addCallback(callback)
+            result.chainDeferred(deferred)
+
+        reactor.callWhenRunning(do_test)
+        return deferred
+
+    def test_run_smart_update_warns_exit_code_1_and_non_empty_stderr(self):
+        """
+        The L{PackageReporter.run_smart_update} method should log a warning
+        in case smart-update terminates with exit code 1 and non empty stderr.
+        """
+        self.reporter.smart_update_filename = self.makeFile(
+            "#!/bin/sh\necho -n \"error  \" >&2\nexit 1")
+        os.chmod(self.reporter.smart_update_filename, 0755)
+        logging_mock = self.mocker.replace("logging.warning")
+        logging_mock("'%s' exited with status 1"
+                     " (error  )" % self.reporter.smart_update_filename)
+        self.mocker.replay()
+        deferred = Deferred()
+        def do_test():
+            result = self.reporter.run_smart_update()
+            def callback((out, err, code)):
+                interval = self.reporter.smart_update_interval
+                self.assertEquals(out, "")
+                self.assertEquals(err, "error  ")
+                self.assertEquals(code, 1)
+            result.addCallback(callback)
+            result.chainDeferred(deferred)
+
+        reactor.callWhenRunning(do_test)
+        return deferred
+
+    def test_run_smart_update_ignores_exit_code_1_and_empty_output(self):
+        """
+        The L{PackageReporter.run_smart_update} method should not log anything
+        in case smart-update terminates with exit code 1 and output containing
+        only a newline character.
+        """
+        self.reporter.smart_update_filename = self.makeFile(
+            "#!/bin/sh\necho\nexit 1")
+        os.chmod(self.reporter.smart_update_filename, 0755)
+        deferred = Deferred()
+        def do_test():
+            def raiseme(x):
+                raise Exception
+            logging.warning = raiseme
+            result = self.reporter.run_smart_update()
+            def callback((out, err, code)):
+                interval = self.reporter.smart_update_interval
+                self.assertEquals(out, "\n")
+                self.assertEquals(err, "")
+                self.assertEquals(code, 1)
+            result.addCallback(callback)
+            result.chainDeferred(deferred)
+
+        reactor.callWhenRunning(do_test)
+        return deferred
+
     def test_remove_expired_hash_id_request(self):
         request = self.store.add_hash_id_request(["hash1"])
         request.message_id = 9999
@@ -569,20 +975,29 @@ class PackageReporterTest(LandscapeIsolatedTest):
 
         self.mocker.order()
 
-        results = [Deferred() for i in range(4)]
+        results = [Deferred() for i in range(7)]
 
-        reporter_mock.handle_tasks()
+        reporter_mock.run_smart_update()
         self.mocker.result(results[0])
 
-        reporter_mock.remove_expired_hash_id_requests()
+        reporter_mock.fetch_hash_id_db()
         self.mocker.result(results[1])
 
-        reporter_mock.request_unknown_hashes()
+        reporter_mock.use_hash_id_db()
         self.mocker.result(results[2])
 
-        reporter_mock.detect_changes()
+        reporter_mock.handle_tasks()
         self.mocker.result(results[3])
 
+        reporter_mock.remove_expired_hash_id_requests()
+        self.mocker.result(results[4])
+
+        reporter_mock.request_unknown_hashes()
+        self.mocker.result(results[5])
+
+        reporter_mock.detect_changes()
+        self.mocker.result(results[6])
+
         self.mocker.replay()
 
         self.reporter.run()
@@ -625,7 +1040,8 @@ class PackageReporterTest(LandscapeIsolatedTest):
     def test_resynchronize(self):
         """
         When a resynchronize task arrives, the reporter should clear
-        out all the data in the package store, except the hash ids.
+        out all the data in the package store, except the hash ids and
+        the hash ids requests.
         This is done in the reporter so that we know it happens when
         no other reporter is possibly running at the same time.
         """
@@ -636,6 +1052,11 @@ class PackageReporterTest(LandscapeIsolatedTest):
         request1 = self.store.add_hash_id_request(["hash3"])
         request2 = self.store.add_hash_id_request(["hash4"])
 
+        # Set the message id to avoid the requests being deleted by the
+        # L{PackageReporter.remove_expired_hash_id_requests} method.
+        request1.message_id = 1
+        request2.message_id = 2
+
         # Let's make sure the data is there.
         self.assertEquals(self.store.get_available_upgrades(), [2])
         self.assertEquals(self.store.get_available(), [1])
@@ -643,7 +1064,7 @@ class PackageReporterTest(LandscapeIsolatedTest):
         self.assertEquals(self.store.get_hash_id_request(request1.id).id, request1.id)
 
         self.store.add_task("reporter", {"type": "resynchronize"})
-        
+
         deferred = self.reporter.run()
 
         def check_result(result):
@@ -659,13 +1080,22 @@ class PackageReporterTest(LandscapeIsolatedTest):
             self.assertEquals(self.store.get_available(), [3, 4])
             self.assertEquals(self.store.get_installed(), [])
 
-            # A New hash id request should also be detected for HASH3,
-            # but there should be no other hash id requests.
-            request = self.store.get_hash_id_request(request1.id)
-            self.assertEquals(request.id, request1.id)
-            self.assertEquals(request.hashes, [HASH3])
-            self.assertRaises(UnknownHashIDRequest,
-                              self.store.get_hash_id_request, request2.id)
+            # The two original hash id requests should be still there, and
+            # a new hash id request should also be detected for HASH3.
+            requests_count = 0
+            new_request_found = False
+            for request in self.store.iter_hash_id_requests():
+                requests_count += 1
+                if request.id == request1.id:
+                    self.assertEquals(request.hashes, ["hash3"])
+                elif request.id == request2.id:
+                    self.assertEquals(request.hashes, ["hash4"])
+                elif not new_request_found:
+                    self.assertEquals(request.hashes, [HASH3])
+                else:
+                    self.fail("Unexpected hash-id request!")
+            self.assertEquals(requests_count, 3)
+
         deferred.addCallback(check_result)
         return deferred
 
