@@ -1,20 +1,35 @@
 import threading
 import time
-import os
+
+try:
+    import sqlite3
+except ImportError:
+    from pysqlite2 import dbapi2 as sqlite3
 
 from landscape.tests.helpers import LandscapeTest
 
-from landscape.package.store import PackageStore, UnknownHashIDRequest
+from landscape.package.store import (HashIdStore, PackageStore,
+                                     UnknownHashIDRequest, InvalidHashIdDb)
 
 
-class PackageStoreTest(LandscapeTest):
+class HashIdStoreTest(LandscapeTest):
 
     def setUp(self):
-        super(PackageStoreTest, self).setUp()
+        super(HashIdStoreTest, self).setUp()
 
         self.filename = self.makeFile()
-        self.store1 = PackageStore(self.filename)
-        self.store2 = PackageStore(self.filename)
+        self.store1 = HashIdStore(self.filename)
+        self.store2 = HashIdStore(self.filename)
+
+    def test_set_and_get_hash_id(self):
+        self.store1.set_hash_ids({"ha\x00sh1": 123, "ha\x00sh2": 456})
+        self.assertEquals(self.store1.get_hash_id("ha\x00sh1"), 123)
+        self.assertEquals(self.store1.get_hash_id("ha\x00sh2"), 456)
+
+    def test_get_hash_ids(self):
+        hash_ids = {"hash1": 123, "hash2": 456}
+        self.store1.set_hash_ids(hash_ids)
+        self.assertEquals(self.store1.get_hash_ids(), hash_ids)
 
     def test_wb_transactional_commits(self):
         mock_db = self.mocker.replace(self.store1._db)
@@ -28,11 +43,6 @@ class PackageStoreTest(LandscapeTest):
         self.mocker.replay()
         self.assertRaises(Exception, self.store1.set_hash_ids, None)
 
-    def test_set_and_get_hash_id(self):
-        self.store1.set_hash_ids({"ha\x00sh1": 123, "ha\x00sh2": 456})
-        self.assertEquals(self.store2.get_hash_id("ha\x00sh1"), 123)
-        self.assertEquals(self.store2.get_hash_id("ha\x00sh2"), 456)
-
     def test_get_id_hash(self):
         self.store1.set_hash_ids({"hash1": 123, "hash2": 456})
         self.assertEquals(self.store2.get_id_hash(123), "hash1")
@@ -70,6 +80,115 @@ class PackageStoreTest(LandscapeTest):
         self.assertTrue(time.time()-started < 5,
                         "Setting 20k hashes took more than 5 seconds.")
 
+    def test_check_sanity(self):
+
+        store_filename = self.makeFile()
+        db = sqlite3.connect(store_filename)
+        cursor = db.cursor()
+        cursor.execute("CREATE TABLE hash"
+                       " (junk INTEGER PRIMARY KEY, hash BLOB UNIQUE)")
+        cursor.close()
+        db.commit()
+
+        store = HashIdStore(store_filename)
+        self.assertRaises(InvalidHashIdDb, store.check_sanity)
+
+
+class PackageStoreTest(LandscapeTest):
+
+    def setUp(self):
+        super(PackageStoreTest, self).setUp()
+
+        self.filename = self.makeFile()
+        self.store1 = PackageStore(self.filename)
+        self.store2 = PackageStore(self.filename)
+
+    def test_has_hash_id_db(self):
+
+        self.assertFalse(self.store1.has_hash_id_db())
+
+        hash_id_db_filename = self.makeFile()
+        HashIdStore(hash_id_db_filename)
+        self.store1.add_hash_id_db(hash_id_db_filename)
+
+        self.assertTrue(self.store1.has_hash_id_db())
+
+    def test_add_hash_id_db_with_non_sqlite_file(self):
+
+        def junk_db_factory():
+            filename = self.makeFile()
+            open(filename, "w").write("junk")
+            return filename
+
+        def raiseme():
+            store_filename = junk_db_factory()
+            try:
+                self.store1.add_hash_id_db(store_filename)
+            except InvalidHashIdDb, e:
+                self.assertEqual(str(e), store_filename)
+            else:
+                self.fail()
+
+        raiseme()
+        self.assertFalse(self.store1.has_hash_id_db())
+
+    def test_add_hash_id_db_with_wrong_schema(self):
+
+        def non_compliant_db_factory():
+            filename = self.makeFile()
+            db = sqlite3.connect(filename)
+            cursor = db.cursor()
+            cursor.execute("CREATE TABLE hash"
+                           " (junk INTEGER PRIMARY KEY, hash BLOB UNIQUE)")
+            cursor.close()
+            db.commit()
+            return filename
+
+        self.assertRaises(InvalidHashIdDb, self.store1.add_hash_id_db,
+                          non_compliant_db_factory())
+        self.assertFalse(self.store1.has_hash_id_db())
+
+    def hash_id_db_factory(self, hash_ids):
+        filename = self.makeFile()
+        store = HashIdStore(filename)
+        store.set_hash_ids(hash_ids)
+        return filename
+
+    def test_get_hash_id_using_hash_id_dbs(self):
+
+
+        # Without hash=>id dbs
+        self.assertEquals(self.store1.get_hash_id("hash1"), None)
+        self.assertEquals(self.store1.get_hash_id("hash2"), None)
+
+        # This hash=>id will be overriden
+        self.store1.set_hash_ids({"hash1": 1})
+
+        # Add a couple of hash=>id dbs
+        self.store1.add_hash_id_db(self.hash_id_db_factory({"hash1": 2,
+                                                            "hash2": 3}))
+        self.store1.add_hash_id_db(self.hash_id_db_factory({"hash2": 4,
+                                                            "ha\x00sh1": 5}))
+
+        # Check look-up priorities and binary hashes
+        self.assertEquals(self.store1.get_hash_id("hash1"), 2)
+        self.assertEquals(self.store1.get_hash_id("hash2"), 3)
+        self.assertEquals(self.store1.get_hash_id("ha\x00sh1"), 5)
+
+    def test_get_id_hash_using_hash_id_db(self):
+        """
+        When lookaside hash->id dbs are used, L{get_id_hash} has
+        to query them first, falling back to the regular db in case
+        the desired mapping is not found.
+        """
+        self.store1.add_hash_id_db(self.hash_id_db_factory({"hash1": 123}))
+        self.store1.add_hash_id_db(self.hash_id_db_factory({"hash1": 999,
+                                                            "hash2": 456}))
+        self.store1.set_hash_ids({"hash3": 789})
+        self.assertEquals(self.store1.get_id_hash(123), "hash1")
+        self.assertEquals(self.store1.get_id_hash(456), "hash2")
+        self.assertEquals(self.store1.get_id_hash(789), "hash3")
+
     def test_add_and_get_available_packages(self):
         self.store1.add_available([1, 2])
         self.assertEquals(self.store2.get_available(), [1, 2])
@@ -326,7 +445,7 @@ class PackageStoreTest(LandscapeTest):
                           self.store1.get_hash_id_request, request1.id)
         self.assertRaises(UnknownHashIDRequest,
                           self.store1.get_hash_id_request, request2.id)
-    
+
     def test_clear_tasks(self):
         data = {"answer": 42}
         task = self.store1.add_task("reporter", data)
@@ -388,4 +507,3 @@ class PackageStoreTest(LandscapeTest):
             thread.join()
 
         self.assertEquals(error, [])
-
